import boto3
import time
import logging

logger = logging.getLogger(__name__)


def invalidate(AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, CLOUDFRONT_DISTRIBUTION_ID, paths=[]):

    client = boto3.client(
        'cloudfront',
        aws_access_key_id=AWS_ACCESS_KEY_ID,
        aws_secret_access_key=AWS_SECRET_ACCESS_KEY
    )

    client.create_invalidation(
        DistributionId=CLOUDFRONT_DISTRIBUTION_ID,
        InvalidationBatch={
            'Paths': {
                'Quantity': len(paths),
                'Items': paths,
            },
            'CallerReference': 'wagtail-invalidate-%s' % time.time()
        }
    )

    logger.info('Created cloudfront invalidation for %s' % paths)




from wagtail.core import hooks
from django.apps import apps
from botocore.exceptions import ParamValidationError
from wagtail.admin import messages
from django.utils.translation import ugettext as _
from wagtail_cloudfront_invalidate import invalidate


@hooks.register('after_edit_page')
def my_hook_function(request, page):

    conf = apps.get_app_config('wagtail_cloudfront_invalidate')

    if not conf.enabled:
        return

    try:

        invalidate(
            conf.AWS_ACCESS_KEY_ID,
            conf.AWS_SECRET_ACCESS_KEY,
            conf.CLOUDFRONT_DISTRIBUTION_ID,
            items=[
                page.get_url(),
            ]
        )

    except ParamValidationError:
        messages.warning(
            request,
            _('Cloudfront Invalidation Error. Check that your AWS access key, secret key and Cloudfront distribution ID are set.')
        )
    except Exception as e:
        messages.warning(
            request,
            e.__str__()
        )




from django.apps import AppConfig
from django.conf import settings
import logging
from wagtail_cloudfront_invalidate import invalidate

logger = logging.getLogger(__name__)


class WagtailCloudfrontInvalidateConfig(AppConfig):
    name = 'wagtail_cloudfront_invalidate'
    verbose_name = "Wagtail Cloudfront Invalidate"

    AWS_ACCESS_KEY_ID = getattr(settings, 'AWS_ACCESS_KEY_ID', None)
    AWS_SECRET_ACCESS_KEY = getattr(settings, 'AWS_SECRET_ACCESS_KEY', None)
    CLOUDFRONT_DISTRIBUTION_ID = getattr(settings, 'CLOUDFRONT_DISTRIBUTION_ID', None)

    @property
    def enabled(self):
        if getattr(settings, 'WAGTAIL_CLOUDFRONT_INVALIDATE_ENABLED', False) is False:
            return False

        return self.has_required_settings

    @property
    def has_required_settings(self):

        for attr in ['AWS_ACCESS_KEY_ID', 'AWS_SECRET_ACCESS_KEY', 'CLOUDFRONT_DISTRIBUTION_ID']:
            if getattr(self, attr) is None:
                logger.info('Please set a value for %s' % attr)
                return False

        return True

    def ready(self):

        if self.enabled:
            try:

                invalidate(
                    self.AWS_ACCESS_KEY_ID,
                    self.AWS_SECRET_ACCESS_KEY,
                    self.CLOUDFRONT_DISTRIBUTION_ID,
                    items=['/*'],
                )

            except Exception as err:
                logger.info(err.__str__())

        else:
            logger.info('Cloudfront invalidate not enabled')



#!/usr/bin/env python
import os
from setuptools import setup, find_packages

setup(
  name='wagtail-cloudfront-invalidate',
  version='1.1.0',
  description="A wagtail hook to invalidate AWS's CloudFront cache when a page is updated",
  long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
  author='Ryan Bagwell',
  author_email='ryan@ryanbagwell.com',
  url='https://github.com/ryanbagwell/wagtail-cloudfront-invalidate/',
  packages=find_packages(),
  classifiers=[
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'Programming Language :: Python',
    'Topic :: Internet :: WWW/HTTP',
  ],
  install_requires=[
    'django>=1.11.0',
    'boto3>=1.9.112',
    'wagtail>=2.0.0',
  ]
)